package com.appconsentclearreactnative

import android.util.Log
import com.facebook.react.bridge.Arguments
import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.ReactContextBaseJavaModule
import com.facebook.react.bridge.ReactMethod
import com.facebook.react.bridge.ReadableMap
import com.facebook.react.bridge.ReadableType
import com.sfbx.appconsentv3.AppConsent
import com.sfbx.appconsentv3.ui.AppConsentSDK
import com.sfbx.appconsentv3.ui.listener.OnPresentNoticeListener
import com.sfbx.appconsentv3.ui.model.ACConfiguration
import com.sfbx.appconsentv3.ui.model.ACConsentStatus
import com.sfbx.appconsentv3.ui.model.ACConsentableType

class AppconsentClearReactnativeModule(private val reactContext: ReactApplicationContext) :
  ReactContextBaseJavaModule(reactContext) {

  override fun getName(): String {
    return "AppconsentClearReactnative"
  }

  private var appconsentUi: AppConsent? = null
  private var appKey: String? = null
  private var forceApplyGDPR: Boolean = false

  override fun getConstants(): MutableMap<String, Any> {
    return mutableMapOf("version" to "5.1.4")
  }

  @ReactMethod
  fun requiresMainQueueSetup(): Boolean {
    return true
  }

  @ReactMethod
  fun configureWith(appKey: String, forceApplyGDPR: Boolean, forceATT: Boolean, promise: Promise) {
    this.appKey = appKey
    this.forceApplyGDPR = forceApplyGDPR
    try {
      AppConsentSDK.initialize(
        appKey = appKey,
        configuration = ACConfiguration.Builder()
          .setForceApplyGDPR(forceApplyGDPR)
          .build()
      ) {
        appconsentUi = it
        promise.resolve(true)
      }
    } catch (ex: Throwable) {
      Log.e(
        tag,
        "An error occurred during the configuration of the module: ${ex.message}",
        ex
      )
      promise.reject(ex)
    }
  }

  @ReactMethod
  fun present(force: Boolean) {
    appconsentUi?.tryToDisplayNotice(force)
  }

  @ReactMethod
  fun checkForUpdate(promise: Promise) {
    appconsentUi?.checkForUpdate(onResult = {
      promise.resolve(it)
    },
      error = {
        promise.reject(it)
      })
  }

  @ReactMethod
  fun isGDPRCountry(promise: Promise) {
    promise.resolve(appconsentUi?.isSubjectToGDPR())
  }

  @ReactMethod
  fun consentAlreadyGiven(promise: Promise) {
    promise.resolve(appconsentUi?.consentGiven())
  }

  @ReactMethod
  fun consentGiven(promise: Promise) {
    appconsentUi?.setOnPresentNoticeListener(object : OnPresentNoticeListener {
      override fun presentConsentError(error: Throwable?) {
        promise.reject(error ?: Throwable("An unknown error has occurred."))
      }

      override fun presentConsentGiven() {
        promise.resolve(appconsentUi?.consentGiven())
      }
    })
  }

  @ReactMethod
  fun clearConsent() {
    appconsentUi?.clearConsent()
  }

  @ReactMethod
  fun extraConsentableAllowed(extraId: String?, promise: Promise) {
    promise.resolve(appconsentUi?.extraConsentableAllowed(extraId!!))
  }

  @ReactMethod
  fun extraVendorAllowed(extraId: String?, promise: Promise) {
    promise.resolve(appconsentUi?.extraVendorAllowed(extraId!!))
  }

  @ReactMethod
  fun extraFloatingPurposeAllowed(extraId: String?, promise: Promise) {
    promise.resolve(appconsentUi?.extraFloatingAllowed(extraId!!))
  }

  @ReactMethod
  fun setConsentableConsent(values: ReadableMap, promise: Promise) {
    val map = values.toHashMap().entries.associate { entry ->
      entry.key.toInt() to ACConsentStatus.values()
        .first { it.ordinal == (entry.value as Double).toInt() }
    }
    appconsentUi?.setConsentableConsents(map,
      success = {
        promise.resolve(true)
      },
      error = {
        promise.reject(it)
      })
  }

  @ReactMethod
  fun setExtraConsentableConsent(values: ReadableMap, promise: Promise) {
    val map = values.toHashMap().mapValues { entry ->
      ACConsentStatus.values().first { it.ordinal == (entry.value as Double).toInt() }
    }
    appconsentUi?.setExtraConsentableConsents(map,
      success = {
        promise.resolve(true)
      },
      error = {
        promise.reject(it)
      }
    )
  }

  // Deprecated
  @ReactMethod
  fun consentableAllowedByObjectId(id: String, promise: Promise) {
    val intId = id.toInt()
    promise.resolve(appconsentUi?.consentableAllowed(intId, ACConsentableType.UNKNOWN))
  }

  @ReactMethod
  fun consentableAllowedByIABId(iabId: String, type: Int, promise: Promise) {
    val intId = iabId.toInt()
    val consentableType: ACConsentableType = when (type) {
      0 -> ACConsentableType.PURPOSE
      1 -> ACConsentableType.FEATURE
      2 -> ACConsentableType.SPECIAL_FEATURE
      3 -> ACConsentableType.SPECIAL_PURPOSE
      else -> ACConsentableType.UNKNOWN
    }
    promise.resolve(appconsentUi?.consentableAllowed(intId, consentableType))
  }

  @ReactMethod
  fun vendorAllowedByIABId(iabId: String, promise: Promise) {
    val intId = iabId.toInt()
    promise.resolve(appconsentUi?.vendorAllowed(intId))
  }

  @ReactMethod
  fun stackAllowedByIABId(iabId: String, promise: Promise) {
    val intId = iabId.toInt()
    promise.resolve(appconsentUi?.stackAllowed(intId))
  }

  @ReactMethod
  fun getAppKey(promise: Promise) {
    promise.resolve(appKey)
  }

  @ReactMethod
  fun getForceApplyGDPR(promise: Promise) {
    promise.resolve(forceApplyGDPR)
  }

  @ReactMethod
  fun getExternalIds(promise: Promise) {
    val externalIds = appconsentUi?.getExternalIds()?.toSortedMap()
    if (externalIds.isNullOrEmpty().not()) {
      promise.resolve(toIosTemplate(externalIds!!))
    } else {
      promise.resolve(toIosTemplate(emptyMap()))
    }
  }

  @ReactMethod
  fun setExternalIds(externalIds: ReadableMap, promise: Promise) {
    if (appconsentUi != null) {
      appconsentUi?.setExternalIds(toMap(externalIds))
      promise.resolve(true)
    } else {
      promise.resolve(false)
    }
  }

  @ReactMethod
  fun saveExternalIds(promise: Promise) {
    appconsentUi?.saveExternalIds({
      promise.resolve(true)
    }, {
      Log.e(
        tag,
        "An error occurred while saving external identifiers !",
        it
      )
      promise.reject(it)
    })
  }

  companion object {
    private val tag = AppconsentClearReactnativeModule::class.java.simpleName

    internal fun toIosTemplate(externalIds: Map<String, String>): String {
      val iosTemplate = StringBuilder()
      if (externalIds.isEmpty()) {
        iosTemplate.append("[:]")
      } else {
        val entries = externalIds.entries.iterator()
        iosTemplate.append("[")
        while (entries.hasNext()) {
          val entry = entries.next()
          iosTemplate.append("\"${entry.key}\": \"${entry.value}\"")
          if (entries.hasNext()) {
            iosTemplate.append(", ")
          }
        }
        iosTemplate.append("]")
      }
      return iosTemplate.toString()
    }

    internal fun toMap(map: ReadableMap?): Map<String, String> {
      val hashmap: MutableMap<String, String> = mutableMapOf()
      val iterator = map!!.keySetIterator()
      while (iterator.hasNextKey()) {
        val key = iterator.nextKey()
        when (map.getType(key)) {
          ReadableType.String -> {
            val value = map.getString(key)
            if (value == null) {
              Log.e(
                tag,
                "You can't pass null as value from: $key."
              )
            } else {
              hashmap[key] = value
            }
          }

          else -> {
            Log.e(
              tag,
              "could not convert object with key: $key."
            )
          }
        }
      }
      return hashmap
    }


    internal fun toReadableMap(values: Map<String, String>): ReadableMap {
      val map = Arguments.createMap()

      val keys = values.keys.iterator()
      while (keys.hasNext()) {
        val key = keys.next()

        map.putString(key, values[key])
      }

      return map
    }
  }

}
